# Created by Matyáš Pokorný on 2018-01-26.

module KXI
	module Application
		# Event handler
		class Event
			# Adds hook to handler
			# @return [Number] Id of hook
			# @yield [sender, *args] Action block of hook
			# @yieldparam sender [Object] Invoker of event
			# @yieldparam args [Array<Object, nil>] Arguments of invocation
			def add(&block)
				@hooks.push(block)
				return @hooks.length - 1
			end
			
			# Removes hook from handler
			# @param id [Number] Id of hook to remove
			def remove(id)
				@hooks.delete_at(id)
			end
			
			# Instantiates the {KXI::Application::Event} class
			def initialize
				@hooks = []
			end
			
			# Invokes all hooks
			# @param sender [Object] Invoker of event
			# @param args [Array<Object, nil>] Arguments of invocation
			def fire(sender, *args)
				@hooks.each { |h| h.call(sender, *args) }
			end
		end
	end
end