# Created by Matyáš Pokorný on 2018-01-20.

module KXI
	module CLI
		# Represents anonymous argument (eg.: verb VALUE)
		class AnonymousArgument < KXI::CLI::Argument
			# Gets default value of argument
			# @return [Object] Default value of argument
			def default
				@def
			end
			
			# Gets whether argument is variadic
			# @return [Bool] True if argument is variadic; false otherwise
			def variadic?
				@var
			end
			
			# Gets syntax of argument
			# @return [String] Syntax of argument
			def syntax
				"#{required? ? '<' : '['}#{name}#{required? ? '>' : ']'}"
			end
			
			# Gets full descriptive name of argument
			# @return [String] Full name of argument
			def headline
				super.upcase
			end
			
			# Instantiates the {KXI::CLI::AnonymousArgument} class
			# @param nm [String] Name of argument
			# @param desc [String] Description of argument
			# @param rq [Bool] Indicates whether argument is required
			# @param df Default value of argument
			# @param var [Bool] Indicates whether argument is variadic
			def initialize(nm, desc, rq = true, df = nil, var = false, &validator)
				super(nm, desc, rq, 5 - (var ? 1 : 0) - (rq ? 0 : 1))
				@def = df
				@var = var
				@val = validator
			end
			
			# Validates value of argument
			# @param val [String, Array<String>] Value of argument
			def validate(val)
				@val.call(val) if @val != nil
			end
		end
	end
end