# Created by Matyáš Pokorný on 2018-01-20.

module KXI
	module CLI
		# Represents argument of command-line interface
		class Argument
			# Gets the name of argument
			# @return [String] Name of argument
			def name
				@name
			end
			
			# Gets syntax of argument
			# @return [String] Syntax of argument
			def syntax
				name
			end
			
			# Gets full descriptive name of argument
			# @return [String] Full name of argument
			def headline
				name
			end
			
			# Gets the description of argument
			# @return [String] Description of argument
			def description
				@desc
			end
			
			# Indicates whether argument is mandatory
			# @return [Bool] True if argument is mandatory; otherwise false
			def required?
				@req
			end
			
			# Gets the order of argument (in descending order)
			# @return [Number] Order of argument
			def order
				@order
			end
			
			# Instantiates the {KXI::CLI::Argument} class
			# @param nm [String] Name of argument
			# @param desc [String] Description of argument
			# @param req [Bool] Determines whether argument is mandatory
			# @param order [Number] Order of argument
			def initialize(nm, desc, req, order)
				raise(Exception.new('Invalid argument name!')) unless /^[A-Za-z0-9\-]+$/m =~ nm
				@name  = nm.downcase
				@desc  = desc
				@req   = req
				@order = order
			end
		end
	end
end