# Created by Matyáš Pokorný on 2018-01-21.

module KXI
	module CLI
		# Manages values of arguments
		class ArgumentValues
			# Instantiates the {KXI::CLI::ArgumentValues} class
			# @param args [Array<KXI::CLI::Argument>] Expected arguments
			def initialize(args)
				@args = args
				@vals = {}
				args.each do |a|
					if a.is_a?(KXI::CLI::FlagArgument)
						@vals[a.name] = { :argument => a, :value => false }
					end
				end
			end
			
			# Assigns (or adds) a value to argument
			# @param arg [KXI::CLI::Argument] Argument to set
			# @param val [String] Value to assign
			def set(arg, val)
				if arg.is_a?(KXI::CLI::FlagArgument)
					raise(KXI::Exceptions::ArgumentException.new(arg.name, 'Flag set multiple times!')) if @vals[arg.name][:value]
					@vals[arg.name][:value] = val
				else
					if arg.variadic?
						@vals[arg.name] = { :argument => arg, :value => [] } if @vals[arg.name] == nil
						@vals[arg.name][:value].push(val)
					else
						raise(KXI::Exceptions::ArgumentException.new(arg.name, 'Argument set multiple times!')) if @vals[arg.name] != nil
						begin
							arg.validate(val)
						rescue Exception => ex
							raise(KXI::Exceptions::ArgumentException.new(arg.name, ex.message))
						end
						@vals[arg.name] = { :argument => arg, :value => val }
					end
				end
			end
			
			# Gets value of argument
			# @param index [String, Symbol, KXI::CLI::Argument] Name of argument
			# @return [String] Value of argument
			def [](index)
				if index.is_a?(Symbol)
					index = index.to_s
				elsif index.is_a?(KXI::CLI::Argument)
					index = index.name
				end
				raise(Exception.new("Undefined argument '#{index}'!")) unless @vals.include?(index)
				return @vals[index][:value]
			end
			
			# Converts class to hash
			# @return [Hash] Equivalent hash
			def to_h
				ret = {}
				@vals.each_pair do |k, v|
					ret[k] = v[:value]
				end
				return ret
			end
			
			# Validates variadic arguments and checks for minimal argument requirements
			def finish
				@args.each do |arg|
					if @vals.include?(arg.name)
						unless arg.is_a?(KXI::CLI::FlagArgument)
							begin
								arg.validate(@vals[arg.name][:value])
							rescue Exception => ex
								raise(KXI::Exceptions::ArgumentException.new(arg.name, ex.message))
							end
						end
					else
						raise(KXI::Exceptions::ArgumentException.new(arg.name, 'Argument is mandatory!')) if arg.required?
						@vals[arg.name] = { :argument => arg, :value => arg.default }
					end
				end
			end
		end
	end
end