# Created by Matyáš Pokorný on 2018-01-20.

module KXI
	module CLI
		# Represents named argument (eg.: -a VALUE, --argument VALUE)
		class NamedArgument < KXI::CLI::ExplicitArgument
			# Gets default value of argument
			# @return [Object] Default value of argument
			def default
				@def
			end
			
			# Gets whether argument is variadic
			# @return [Bool] True if argument is variadic; false otherwise
			def variadic?
				@var
			end
			
			# Gets name of argument value
			# @return [String] Name of argument value
			def value_name
				@vnm
			end
			
			# Gets full descriptive name of argument
			# @return [String] Full name of argument
			def headline
				"#{super} #{@var ? '...' : ''}#{@vnm.upcase}"
			end
			
			# Gets syntax of argument
			# @return [String] Syntax of argument
			def syntax
				"-#{(shortcut != nil ? shortcut : "-#{name}")} #{required? ? '<' : '['}#{@var ? '...' : ''}#{@vnm}#{required? ? '>' : ']'}"
			end
			
			# Instantiates the {KXI::CLI::NamedArgument} class
			# @param nm [String] Name of argument
			# @param vn [String] Name of value
			# @param desc [String] Description of argument
			# @param sh [String] Shortcut of argument
			# @param rq [Bool] Indicates whether argument is required
			# @param df Default value of argument
			# @param var [Bool] Indicates whether argument is variadic
			def initialize(nm, vn, desc, sh = nil, rq = true, df = nil, var = false, &validator)
				super(nm, desc, sh, rq)
				@def = df
				@vnm = vn
				@var = var
				@val = validator
			end
			
			# Validates value of argument
			# @param val [String, Array<String>] Value of argument
			def validate(val)
				@val.call(val) if @val != nil
			end
		end
	end
end