# Created by Matyáš Pokorný on 2018-01-24.

module KXI
	module CLI
		# Property list renderer
		class PropertyList
			# Instantiates the {KXI::CLI::PropertyList} class
			def initialize
				@align = 0
				@data = {}
			end
			
			# Sets a property
			# @param [string] name Name of the property
			# @param [string,Array<string>] value Value of the property
			def field(name, value)
				@data[name] = value
				@align = name.length if name.length > @align
			end
			
			# Renders the property list into stdout
			# @param [integer] cols Determines the maximal number of columns for array rendering
			def render(cols = 4)
				@data.each_pair do |k, v|
					if v.kind_of?(Array)
						print("#{pad(k)}: ")
						v.each_index do |idx|
							if idx > 0 and idx % cols == 0
								puts('')
								print(' ' * (@align + 2))
							elsif idx > 0
								print(', ')
							end
							print(v[idx])
						end
						puts('')
					else
						puts("#{pad(k)}: #{v}")
					end
				end
			end
			
			# Adds a space-dot-space padding
			# @param [string] key Key to add padding to
			def pad(key)
				d = false
				p = ''
				(@align - key.length).times do |i|
					p = (d ? '.' : ' ') + p
					d = (not d)
				end
				return key + p
			end
			
			private :pad
		end
	end
end