# Created by Matyáš Pokorný on 2018-01-24.

module KXI
	module CLI
		# Table renderer
		class Table
			# Instantiate the {KXI::CLI::Table} class
			def initialize
				@cols = KXI::Collections::ArrayCollection.new
				@rows = 0
				yield(self) if block_given?
			end
			
			# Defines a column of table
			# @param [string] name Name of column
			# @return [KXI::CLI::Table::Column] Defined column
			def column(name)
				col = Column.new(name)
				@cols.add(col)
				return col
			end
			
			# Adds a row to table
			# @param [Hash<string, any>] e Row to add
			# @return [integer] Current number of rows
			def row(e)
				e.each_key { |c| @cols.first { |i| i.name == c }&.row(e[c].to_s.strip.chomp) }
				@rows += 1
			end
			
			# Renders the table into stdout
			def render
				@rows.times do |row|
					f = true
					@cols.foreach do |col|
						print(' ') unless f
						f = false
						col.write(row)
					end
					puts
				end
			end
			
			# Represents a column of table
			class Column
				# Gets the length of the longest value
				# @return [integer] Length of the longest value
				def align
					@align
				end
				
				# Gets the name of column
				# @return [string] Name of column
				def name
					@name
				end
				
				# Instantiates the {KXI::CLI::Table::Column} class
				# @param [string] name Name of column
				def initialize(name)
					@name  = name
					@align = 0
					@data  = []
				end
				
				# Adds a row to the column
				# @param [string] str Value of row to add
				def row(str)
					@align = str.length if @align < str.length
					@data.push(str)
				end
				
				# Writes the value of a row at given index to stdout
				# @param [integer] row Index of the row to write
				def write(row)
					str = @data[row]
					str = '' if str == nil
					print(str.ljust(@align))
				end
			end
		end
	end
end