# Created by Matyáš Pokorný on 2017-12-29.

module KXI
	module Collections
		# Makes hash enumerable.
		# Every key-value pair of hash is represented by {KXI::Collections::HashCollection::HashEnumerator::KeyValuePair} class
		class HashCollection < KXI::Collections::Enumerable
			# Instantiates the {KXI::Collections::HashCollection} class
			# @param hash [Hash] Hash for enumeration
			def initialize(hash)
				super()
				@hash = hash
			end
			
			# Creates a new {KXI::Collections::Enumerator} bound to this instance
			# @return [KXI::Collections::Enumerator] Enumerator bound to this instance
			def create_enumerator
				HashEnumerator.new(@hash)
			end
			
			# Gets value from hash at given key
			# @param key [Object] Key of value to obtain
			# @return [Object] Obtained value
			def [](key)
				lock do
					return @hash[key]
				end
			end
			
			# Sets value to hash at given key
			# @param key [Object] Key of pair to set
			# @param value [Object] Value to set the pair to
			# @return [Object] Given value
			def []=(key, value)
				lock(true) do
					@hash[key] = value
				end
				return value
			end
			
			# Enumerates hash as key-value pairs represented by the {KXI::Collections::HashCollection::HashEnumerator::KeyValuePair} class
			class HashEnumerator < KXI::Collections::Enumerator
				# Instantiates the {KXI::Collections::HashCollection::HashEnumerator} class
				# @param hash [Hash] Hash for enumeration
				def initialize(hash)
					@hash = hash
					@keys = hash.keys
					@current = 0
				end
				
				# Selects first item in collection
				# @return [Bool] True if collection contains elements; otherwise false
				def rewind
					@current = 0
					return @keys.length > 0
				end
				
				# Advances enumerator to next item
				# @return [Bool] True if item is available; false otherwise
				# @raise [KXI::Exceptions::AbstractException] When method is not implemented in superclass
				def next
					@current += 1
					return @keys.length > @current
				end
				
				# Returns current item
				# @return [Object] Current item
				# @raise [KXI::Exceptions::AbstractException] When method is not implemented in superclass
				def current
					k = @keys[@current]
					KeyValuePair.new(k, @hash[k])
				end
				
				# Represents a key-value pair
				class KeyValuePair
					# Returns the key of pair
					# @return [Object] Key of pair
					def key
						@key
					end
					
					# Returns the value of pair
					# @return [Object] Value of pair
					def value
						@val
					end
					
					# Instantiates the {KXI::Collections::HashCollection::HashEnumerator::KeyValuePair} class
					# @param k [Object] Key of pair
					# @param v [Object] Value of pair
					def initialize(k, v)
						@key = k
						@val = v
					end
				end
			end
			
			protected :create_enumerator
		end
	end
end