# Created by Matyáš Pokorný on 2017-12-29.

module KXI
	module Exceptions
		# Raised when certain value is out of range
		class OutOfRangeException < Exception
			# Returns erroneous value
			# @return [Object] Erroneous value
			def value
				@val
			end
			
			# Returns minimal expected value
			# @return [Object, nil] Minimal expected value; nil if there is no minimum
			def minimum
				@min
			end
			
			# Returns maximal expected value
			# @return [Object, nil] Maximal expected value; nil if there is no maximum
			def maximum
				@max
			end
			
			# Instantiates the {KXI::Exceptions::OutOfRangeException} class
			# @param val [Object] Erroneous value
			# @param min [Object, nil] Minimal expected value; nil if there is no minimum
			# @param max [Object, nil] Maximal expected value; nil if there is no maximum
			# @note If both min and max are nil, then range is empty set
			def initialize(val, min = nil, max = nil)
				if min == nil and max == nil
					super("Value '#{val}' is out of range {}!")
				elsif min == max
					super("Value '#{val}' is out of range {#{min}}!")
				else
					super("Value '#{val}' is out of range #{(min == nil ? '(-∞' : "<#{min}")};#{(max == nil ? '∞)' : "#{max}>")}!")
				end
				@val = val
				@min = min
				@max = max
			end
		end
	end
end