# Created by Matyáš Pokorný on 2018-04-12.

module KXI
	module Math
		# Computes factorial
		# @param [Integer] n Factorial base
		# @return [Integer] Value of factorial
		def self.fact(n)
			n <= 0 ? 1 : n * fact(n - 1)
		end
		
		def self.lfact(n, l)
			return (l == 1 or n <= 0) ? n : n * lfact(n - 1, l - 1)
		end
		
		# Computes fraction of factorials (!n / !m)
		# @param [Integer] n Factorial base of numerator
		# @param [Integer] m Factorial base of denominator
		# @return [Numeric] Result of fraction
		def self.pfact(n, m)
			return 1 if n == m
			return (1.to_f / lfact(m, m - n)) if m > n
			return lfact(n, n - m)
		end
		
		# Computes the binomial coefficient
		# @param [Integer] n First (upper) term of coefficient
		# @param [Integer] k Second (lower) term of coefficient
		# @return [Numeric] Binomial coefficient (n choose k)
		def self.choose(n, k)
			return pfact(n, k) / fact(n - k)
		end
		
		# Computes the permutation of two numbers
		# @param [Integer] n First term of permutation
		# @param [Integer] k Second term of permutation
		# @return [Numeric] Permutation (k-permutations of n)
		def self.permutation(n, k)
			return 0 if k > n
			pfact(n, n - k)
		end
		
		private_class_method :lfact
	end
end