# Created by Matyáš Pokorný on 2018-01-28.

module KXI
	# Represents an platform of execution
	class Platform
		@@this = nil
		# Microsoft Windows
		WINDOWS = 'windows'
		# Apple macOS
		OSX = 'osx'
		# Linux-based system
		LINUX = 'linux'
		
		# Gets the name of platform
		# @return [string] Name of platform
		def name
			@name
		end
		
		# Gets the version of platform
		# @return [KXI::Application::Version] Version of platform
		def version
			@version
		end
		
		# Gets the file extensions of libraries
		# @return [Array<string>] File extensions of libraries
		def library_extensions
			@libs
		end
		
		# Gets the separator of paths
		# @return [string] Separator of paths
		def path_separator
			@ps
		end
		
		# Gets the separator of directories within a path
		# @return [string] Separator of directories
		def directory_separator
			@ds
		end
		
		# Determines whether paths are case-sensitive
		# @return [bool] True if paths are case-sensitive; false otherwise
		def path_case_sensitive?
			@cs
		end
		
		# Instantiates the {KXI::Platform} class
		# @param [string] nm Name of platform
		# @param [string] ver Version of platform
		# @param [string] ds Directory separator
		# @param [string] ps Path separator
		# @param [string] cs Determines whether paths are case-sensitive
		# @param [string] libs File extensions of libraries
		def initialize(nm, ver, ds, ps, cs, *libs)
			@name    = nm
			@version = KXI::Application::Version.parse(ver)
			@ds      = ds
			@ps      = ps
			@cs      = cs
			@libs    = libs
		end
		
		# Gets the current platform
		# @return [KXI::Platform] Current platform
		def self.this
			return @@this if @@this != nil
			if /cygwin|mswin|mingw|bccwin|wince|emx/ =~ RUBY_PLATFORM
				@@this = Platform.new(WINDOWS, `wmic os get Version /value`.split('=').last.strip.chomp, '\\', ';', false, 'dll')
			elsif /darwin/ =~ RUBY_PLATFORM
				@@this = Platform.new(OSX, `defaults read loginwindow SystemVersionStampAsString`, '/', ':', true, 'dylib')
			else
				@@this = Platform.new(LINUX, `uname -r`.split('-')[0], '/', ':', true, 'so')
			end
			return @@this
		end
		
		# Finds an executable within the PATH of platform
		# @return [string,nil] Path of executable if found; otherwise nil
		def exec(name, *dir)
			dir = Platform.env('path').split(@ps) if dir.length == 0
			ext = Platform.env('pathext')
			(ext != nil ? ext.split(@ps) + [''] : ['']).each do |e|
				dir.each do |d|
					f = File.join(d, "#{name}#{e}")
					return f if File.exists?(f)
				end
			end
			return nil
		end
		
		# Gets an environment variable
		# @return [string] Value of environment variable if found; otherwise nil
		def self.env(key)
			key = key.downcase
			ENV.each_pair do |k, v|
				return v if k.downcase == key
			end
			return nil
		end
	end
end