# Created by Matyáš Pokorný on 2017-12-28.
module KXI
	module Reflection
		# Represents frame of call stack
		class StackFrame
			# Returns the file of frame
			# @return [String] File of frame
			def file
				@file
			end
			
			# Returns the line number of frame
			# @return [Number] Line number of frame
			def line
				@line
			end
			
			# Returns the context of frame
			# @return [String] Context of frame
			def context
				@context
			end
			
			# Indicates whether frame is block
			# @return [Bool] True if stack frame represents call to block; false otherwise
			def block?
				@block != nil
			end
			
			# Returns containing method of frame; nil if none
			# @return [String, nil] Name of method that the stack frame represents
			def method
				return nil if @block != nil and @block[0] == '<' and @block[@block.length - 1] == '>'
				return @block unless @block == nil
				return nil if @context != nil and @context[0] == '<' and @context[@context.length - 1] == '>'
				@context
			end
			
			# Instantiates the {KXI::Reflection::StackFrame} class
			# @param file [String] File of frame
			# @param line [Number] Line number of frame
			# @param context [String, nil] Context of frame
			def initialize(file, line, context = nil)
				@file    = file
				@line    = line
				@context = context
				if context == nil
					@block = nil
				else
					mt     = /^block (\((?'levels'\d+) levels\) )?in (?'method'.+)$/mi.match(context)
					@block = mt == nil ? nil : mt['method']
				end
			end
			
			# Converts class to string
			# @return [String] Equivalent string
			def to_s
				"#{@file}:#{@line}#{(@context != nil ? ": in `#{@context}'" : '')}"
			end
			
			# Parses callstack into array of {KXI::Reflection::StackFrame}
			# @param skip [Number] Number of callstack records to ignore
			# @return [Array<KXI::Reflection::StackFrame>] Equivalent array of stack frames
			def self.callstack(skip = 1)
				ret = []
				caller(skip).each do |bt|
					ret.push(from_backtrace(bt))
				end
				return ret
			end
			
			# Parses backtrace line into {KXI::Reflection::StackFrame}
			# @param bt [String] Backtrace line
			# @return [KXI::Reflection::StackFrame] Equivalent stack frame
			def self.from_backtrace(bt)
				mt = /(?'file'.+?):(?'line'\d+)(:\s*in\s*`(?'context'.+?)')?/.match(bt)
				StackFrame.new(mt['file'], mt['line'].to_i, mt['context'])
			end
		end
	end
end